// @ myth - common - list.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// Define a generic list class

// @@ include

#include "common/list.h"
#include "common/error.h"

// @@ LIST

// @@@ void LIST::reset(int target = 0)

void LIST::reset(int target = 0)
{
    if(target >= links)
    {
	previous = last;
	pointer = NULL;
	left = 0;
    }
    else
    {
	if(target < where())
	{
	    previous = NULL;
	    pointer = list;
	    left = links;
	}
	while(target > where())
	{
	    previous = pointer;
	    pointer = pointer->next;
	    left--;
	}
    }
}
    

// @@@ void LIST::add(void* item)

void LIST::add(void* item)
{
    list = new item_link(item, list);
    links++;
    if(pointer == list->next)
	previous = list;
    if(last == NULL)
	last = list;
}

// @@@ void LIST::insert(void* item)

void LIST::insert(void* item)
{
    if(previous == NULL)
	add(item);
    else if(pointer == NULL)
    {
	append(item);
	next();
    }
    else
    {
	previous = previous->next = new item_link(item, pointer);
	links++;
    }
}

// @@@ void LIST::append(void* item)

void LIST::append(void* item)
{
    if(links == 0)
	list = pointer = last = new item_link(item, NULL);
    else
    {
	last->next = new item_link(item, NULL);
	last = last->next;

	if(pointer == NULL)
	    pointer = last;
    }
    links++;
    left++;
}

// @@@ void LIST::sort(pfEpVpV compare)
//
// Lets try a merge sort.

void LIST::sort(pfEpVpV compare)
{
    if(length() < 2)
	return;
    
    LIST merge[2];
    
    reset();
    while(remain() > 0)
    {
	merge[remain() % 2].append(current());

	remove();
    }
    merge[0].sort(compare);
    merge[1].sort(compare);

    merge[0].reset();
    merge[1].reset();

    while(merge[0].length() > 0 || merge[1].length() > 0)
    {
	int which;
	
	if(merge[0].length() < 1)
	    which = 1;
	else if(merge[1].length() < 1)
	    which = 0;
	else if(compare(merge[0].current(), merge[1].current()) == LESS_THAN)
	    which = 1;
	else
	    which = 0;

	append(merge[which].current());
	merge[which].remove();
    }
}

// @@@ void LIST::clear()

void LIST::clear()
{
    reset();
    
    while(current() != NULL)
	remove();
}

// @@@ void LIST::remove()

void LIST::remove()
{
    // We must have something to remove
    if(pointer == NULL)
    {
	bug.error("LIST::remove", "NULL pointer");
	return;
    }
    // If we remove the last item, the previous item become the last!
    if(pointer == last)
	last = previous;
    // Go to next item
    pointer = pointer->next;
    
    if(previous == NULL)
    {
	// If we deleted the first item, the next item become the first!
	delete list;
	list = pointer;
    }
    else
    {
	// Else remove the item from the list
	delete previous->next;
	previous->next = pointer;
    }
    // And update the counts
    left--;
    links--;
}

// @@@ LIST::LIST()

LIST::LIST()
{
    list = pointer = last = previous = NULL;
    links = left= 0;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
