// @ myth - common - parse.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef PARSE_HEADER
#define PARSE_HEADER

// @@ include

#include "common/common.h"
#include "common/string.h"
#include "common/buffer.h"
#include "common/range.h"

// @@ enum input_type

enum input_type
{ INPUT_WORD, INPUT_RANGE, INPUT_LINE, INPUT_LONG, INPUT_SPACE, INPUT_WHITE,
  INPUT_NEWLINE, INPUT_COMMENT, INPUT_KEY, INPUT_STRING };

// @@ class PARSE

class PARSE : BUFFER
{
    int NextChar;
    char buffer[MessageSize];
    int CurrentLine;
    STRING FileName;
public:
    void wrong(const char* where, const char* what);
    void wrong(const char* where, STRING what);
    char get();
    void put(char c);
    bool LookAhead(input_type type, char c = EOF);
    bool LookAhead(const char* s);
    void skip(input_type type);
    void skip(const char* s);
    char* GetWord();
    range GetRange();
    char* GetLine();
    long GetLong();
    char GetKey();
    char* GetString();
    inline bool good()
    { return BUFFER::GoodInput(); }
    inline bool eof()
    { return BUFFER::eof(); }
    PARSE(STRING name);
};

// @@ end

#endif PARSE_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
