// @ myth - server - ability.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef ABILITY_HEADER
#define ABILITY_HEADER

// @@ include

#include "common/common.h"

struct BINARY;
struct LEVEL;
struct MONSTER;

// @@ enum ability_type

enum ability_type
{
    ABILITY_STONED,
    ABILITY_INFECTED,
    ABILITY_PHASING,
    ABILITY_SICK,
    ABILITY_CONFUSED,
    ABILITY_BLIND,
    ABILITY_SIGHT,
    ABILITY_SEE_INVISIBLE,
    ABILITY_INVISIBLE,
    ABILITY_DETECT,
    ABILITY_FAST,
    ABILITY_SLOW,
    ABILITY_LIGHT,
    ABILITY_TELEPORT_CONTROL,
    ABILITY_RANDOM_TELEPORT,
    ABILITY_HEALTH,
    ABILITY_HUNGER,
    ABILITY_STRENGTH,
    ABILITY_YENDOR,
    ABILITY_FIRE_PROTECTION,
    ABILITY_COLD_PROTECTION,
    ABILITY_ACID_PROTECTION,
    ABILITY_STONE_PROTECTION,
    ABILITY_POISON_PROTECTION,
    ABILITY_DEATH_PROTECTION,
    ABILITY_HISS,
    ABILITY_BREATHE_FIRE,
    ABILITY_END,
    ABILITY_ERROR = -1
};

// @@ COUNTER

// @@@ COUNTER

class COUNTER
{
protected:
    long count;			// Number of tick's left
    bool permanent;		// Is it permanent?
    bool done;			// Is it removed by force?
    int MyValue;		// Value for permanent
    int MyAbility;		// What ability does it implement?
    bool touched;		// Changed since last update?
    virtual void effect(LEVEL*, MONSTER*) { }
public:
    ability_type ability()
    { return MyAbility; }
    virtual int value();
    virtual void add(COUNTER* counter);
    virtual cont tick(LEVEL* level, MONSTER* player);
    virtual void remove(force flag);
    virtual void save(BINARY* binary);
    virtual void load(BINARY* binary);
    COUNTER(ability_type ability, long cnt, int val = 0);
};

// @@@ COUNTER_STONED

class COUNTER_STONED : public COUNTER
{
    cont tick(LEVEL*, MONSTER* monster);
public:
    COUNTER_STONED(long c) : (ABILITY_STONED, (c == 0) ? 5L : c) { }
};

// @@@ COUNTER_INFECTED

class COUNTER_INFECTED : public COUNTER
{
    void effect(LEVEL* level, MONSTER* monster);
public:
    COUNTER_INFECTED(long c) : (ABILITY_INFECTED, c) { }
};

// @@@ COUNTER_PHASING

class COUNTER_PHASING : public COUNTER
{
    cont tick(LEVEL*, MONSTER* monster);
public:
    COUNTER_PHASING(long c) : (ABILITY_PHASING, c) { }
    COUNTER_PHASING() : (ABILITY_PHASING, 0L) { }
};

// @@@ COUNTER_SICK

class COUNTER_SICK : public COUNTER
{
    void effect(LEVEL* level, MONSTER* monster);
public:
    COUNTER_SICK(long c) : (ABILITY_SICK, c) { }
};

// @@@ COUNTER_CONFUSED

class COUNTER_CONFUSED : public COUNTER
{
    int value();
    void effect(LEVEL*, MONSTER* monster);
public:
    COUNTER_CONFUSED(long c, int val = 0) : (ABILITY_CONFUSED, c, val) { }
    COUNTER_CONFUSED() : (ABILITY_CONFUSED, 0L) { }
};

// @@@ COUNTER_BLIND

class COUNTER_BLIND : public COUNTER
{
    void effect(LEVEL*, MONSTER* monster);
public:
    COUNTER_BLIND(long c) : (ABILITY_BLIND, c) { }
    COUNTER_BLIND() : (ABILITY_BLIND, 0L) { }
};

// @@@ COUNTER_SIGHT

class COUNTER_SIGHT : public COUNTER
{
    int value();
public:
    COUNTER_SIGHT(long c) : (ABILITY_SIGHT, c) { }
    COUNTER_SIGHT() : (ABILITY_SIGHT, 0L) { }
};

// @@@ COUNTER_SEE_INVISIBLE

class COUNTER_SEE_INVISIBLE : public COUNTER
{
public:
    COUNTER_SEE_INVISIBLE(long c) : (ABILITY_SEE_INVISIBLE, c) { }
    COUNTER_SEE_INVISIBLE() : (ABILITY_SEE_INVISIBLE, 0L) { }
};

// @@@ COUNTER_INVISIBLE

class COUNTER_INVISIBLE : public COUNTER
{
    void effect(LEVEL*, MONSTER* monster);
public:
    COUNTER_INVISIBLE(long c) : (ABILITY_INVISIBLE, c) { }
    COUNTER_INVISIBLE() : (ABILITY_INVISIBLE, 0L) { }
};

// @@@ COUNTER_DETECT

class COUNTER_DETECT : public COUNTER
{
public:
    COUNTER_DETECT(long c) : (ABILITY_DETECT, c) { }
    COUNTER_DETECT() : (ABILITY_DETECT, 0L) { }
};

// @@@ COUNTER_FAST

class COUNTER_FAST : public COUNTER
{
    int value();
    void effect(LEVEL*, MONSTER* monster);
public:
    COUNTER_FAST(long c, int val = 0) : (ABILITY_FAST, c, val) { }
    COUNTER_FAST() : (ABILITY_FAST, 0L) { }
};

// @@@ COUNTER_SLOW

class COUNTER_SLOW : public COUNTER
{
    void effect(LEVEL*, MONSTER* monster);
public:
    COUNTER_SLOW(long c) : (ABILITY_SLOW, c) { }
    COUNTER_SLOW() : (ABILITY_SLOW, 0L) { }
};

// @@@ COUNTER_LIGHT

class COUNTER_LIGHT : public COUNTER
{
    void effect(LEVEL*, MONSTER* monster);
public:
    COUNTER_LIGHT(long c) : (ABILITY_LIGHT, c) { }
    COUNTER_LIGHT() : (ABILITY_LIGHT, 0L) { }
};

// @@@ COUNTER_TELEPORT_CONTROL

class COUNTER_TELEPORT_CONTROL : public COUNTER
{
public:
    COUNTER_TELEPORT_CONTROL(long c) : (ABILITY_TELEPORT_CONTROL, c) { }
    COUNTER_TELEPORT_CONTROL() : (ABILITY_TELEPORT_CONTROL, 0L) { }
};

// @@@ COUNTER_RANDOM_TELEPORT

class COUNTER_RANDOM_TELEPORT : public COUNTER
{
public:
    COUNTER_RANDOM_TELEPORT(long c, int val) :
                           (ABILITY_RANDOM_TELEPORT, c, val) { }
    COUNTER_RANDOM_TELEPORT() : (ABILITY_RANDOM_TELEPORT, 0L) { }
};

// @@@ COUNTER_HEALTH

class COUNTER_HEALTH : public COUNTER
{
public:
    COUNTER_HEALTH(long c, int val) : (ABILITY_HEALTH, c, val) { }
    COUNTER_HEALTH() : (ABILITY_HEALTH, 0L) { }
};

// @@@ COUNTER_HUNGER

class COUNTER_HUNGER : public COUNTER
{
public:
    COUNTER_HUNGER(long c, int val) : (ABILITY_HUNGER, c, val) { }
    COUNTER_HUNGER() : (ABILITY_HUNGER, 0L) { }
};

// @@@ COUNTER_STRENGTH

class COUNTER_STRENGTH : public COUNTER
{
public:
    COUNTER_STRENGTH(long c, int val = 0) : (ABILITY_STRENGTH, c, val) { }
    COUNTER_STRENGTH() : (ABILITY_STRENGTH, 0L) { }
};

// @@@ COUNTER_YENDOR

class COUNTER_YENDOR : public COUNTER
{
public:
    COUNTER_YENDOR(long c) : (ABILITY_YENDOR, c) { }
    COUNTER_YENDOR() : (ABILITY_YENDOR, 0L) { }
};

// @@@ COUNTER_FIRE_PROTECTION

class COUNTER_FIRE_PROTECTION : public COUNTER
{
public:
    COUNTER_FIRE_PROTECTION(long c) : (ABILITY_FIRE_PROTECTION, c) { }
    COUNTER_FIRE_PROTECTION() : (ABILITY_FIRE_PROTECTION, 0L) { }
};

// @@@ COUNTER_COLD_PROTECTION

class COUNTER_COLD_PROTECTION : public COUNTER
{
public:
    COUNTER_COLD_PROTECTION(long c) : (ABILITY_COLD_PROTECTION, c) { }
    COUNTER_COLD_PROTECTION() : (ABILITY_COLD_PROTECTION, 0L) { }
};

// @@@ COUNTER_ACID_PROTECTION

class COUNTER_ACID_PROTECTION : public COUNTER
{
public:
    COUNTER_ACID_PROTECTION(long c) : (ABILITY_ACID_PROTECTION, c) { }
    COUNTER_ACID_PROTECTION() : (ABILITY_ACID_PROTECTION, 0L) { }
};

// @@@ COUNTER_STONE_PROTECTION

class COUNTER_STONE_PROTECTION : public COUNTER
{
public:
    COUNTER_STONE_PROTECTION(long c) : (ABILITY_STONE_PROTECTION, c) { }
    COUNTER_STONE_PROTECTION() : (ABILITY_STONE_PROTECTION, 0L) { }
};

// @@@ COUNTER_POISON_PROTECTION

class COUNTER_POISON_PROTECTION : public COUNTER
{
public:
    COUNTER_POISON_PROTECTION(long c) : (ABILITY_POISON_PROTECTION, c) { }
    COUNTER_POISON_PROTECTION() : (ABILITY_POISON_PROTECTION, 0L) { }
};

// @@@ COUNTER_DEATH_PROTECTION

class COUNTER_DEATH_PROTECTION : public COUNTER
{
public:
    COUNTER_DEATH_PROTECTION(long c) : (ABILITY_DEATH_PROTECTION, c) { }
    COUNTER_DEATH_PROTECTION() : (ABILITY_DEATH_PROTECTION, 0L) { }
};

// @@@ COUNTER_HISS

class COUNTER_HISS : public COUNTER
{
public:
    COUNTER_HISS(long c) : (ABILITY_HISS, c) { }
    COUNTER_HISS() : (ABILITY_HISS, 0L) { }
};

// @@@ COUNTER_BREATHE_FIRE

class COUNTER_BREATHE_FIRE : public COUNTER
{
public:
    COUNTER_BREATHE_FIRE(long c) : (ABILITY_BREATHE_FIRE, c) { }
    COUNTER_BREATHE_FIRE() : (ABILITY_BREATHE_FIRE, 0L) { }
};

// @@ extern

extern COUNTER* LoadCounter(BINARY* binary);
extern direction ConfuseDirection(direction dir);

// @@ COUNTER_LIST

class COUNTER_LIST
{
    COUNTER* table[ABILITY_END];
    LIST list;
public:
    void insert(COUNTER* counter);
    void update(LEVEL* level, MONSTER* monster);
    bool check(ability_type ability)
    { return table[ability] != NULL; }
    int value(ability_type ability)
    { return check(ability) ? table[ability]->value() : 0; }
    void remove(ability_type ability, force flag);
    void clear();
    void save(BINARY* binary);
    void load(BINARY* binary);
    COUNTER_LIST();
    ~COUNTER_LIST();
};

// @@ end

#endif ABILITY_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
