// @ myth - server - bunch.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/bunch.h"
#include "server/player.h"
#include "common/error.h"
#include "common/page.h"
#include "server/rnd.h"
#include "common/format.h"

// @@ BUNCH

// @@@ void BUNCH::MoveUp(LEVEL* level, MONSTER* monster)

void BUNCH::MoveUp(LEVEL* level, MONSTER* monster)
{
    reset();
    while(current() != NULL)
    {
	if(current()->MoveUp(level, monster))
	    return;

	next();
    }
    monster->message("No way up.");
}

// @@@ void BUNCH::MoveDown(LEVEL* level, MONSTER* monster)

void BUNCH::MoveDown(LEVEL* level, MONSTER* monster)
{
    reset();
    while(current() != NULL)
    {
	if(current()->MoveDown(level, monster))
	    return;

	next();
    }
    monster->message("No way down.");
}

// @@@ bool BUNCH::AcceptEnter(MONSTER* monster)

bool BUNCH::AcceptEnter(MONSTER* monster)
{
    reset();
    while(current() != NULL)
    {
	if(!current()->AcceptEnter(monster))
	    return FALSE;

	next();
    }
    return TRUE;
}    

// @@@ bool BUNCH::AcceptLeave(MONSTER* monster)

bool BUNCH::AcceptLeave(MONSTER* monster)
{
    reset();
    while(current() != NULL)
    {
	if(!current()->AcceptLeave(monster))
	    return FALSE;

	next();
    }
    return TRUE;
}    

// @@@ QUESTION* BUNCH::GetQuestion(PLAYER* player, LEVEL* level, coor pos)

QUESTION* BUNCH::GetQuestion(PLAYER* player, LEVEL* level, coor pos)
{
    QUESTION* question;

    reset();
    while(current() != NULL)
    {
	question = current()->GetQuestion(player, level, pos);
	
	if(question != NULL)
	    return question;

	next();
    }
    // if(blocking())
    //      return new QUESTION_GET_FROM_BUNCH(pos);
    return NULL;
}

// @@@ void BUNCH::defend(MONSTER* monster, LEVEL* level)

void BUNCH::defend(MONSTER* monster, LEVEL* level)
{
    reset();
    while(current() != NULL)
    {
	if(current()->defend(monster, level) == DONE)
	    return;

	next();
    }
}    

// @@@ void BUNCH::update(LEVEL* level, coor pos, MONSTER* monster)

void BUNCH::update(LEVEL* level, coor pos, MONSTER* monster)
{
    long size = 0L;
    char MonsterLook = SYMBOL_ERROR;
    
    reset();
    while(current() != NULL)
    {
	if(current()->IsMonster() && current()->size() > size)
	{
	    size = current()->size();
	    MonsterLook = current()->look();
	}
	next();
    }
    INVENTORY::update(level, pos, monster);
}

// @@@ void BUNCH::describe(rock_type rock, PLAYER* player)

void BUNCH::describe(rock_type rock, PLAYER* player)
{
    if(IsRock(rock) && player->VerboseWall())
	player->message(format("There's some %s there.", RockName(rock)));
    switch(length())
    {
    case 0:
	break;
    case 1:
	reset();
	player->message(format("That is %s.",
			       print(current(), PRINT_OBJECT, player)));
	break;
    default:
	{ 
	    PAGE page;
	    CreatePage(&page, player);
	    player->PageMessage("There is:", "> ", &page);
	}
    }
    if(blocking())
	player->message("The place is used up.");
}

// @@@ OBJECT* BUNCH::HitOne()
//
// Select a candiate for a blow.
//
// Choose among the objects with property MightHit(), so the largest
// objects is the most likely candidates.

OBJECT* BUNCH::HitOne()
{
    OBJECT_LIST list;		// Candidates to a blow
    long size = 0;		// Accumulated size

    reset();
    while(current() != NULL)
    {
	if(current()->MightHit())
	{
	    list.add(current());
	    size += current()->size();
	}
	next();
    }

    list.reset();
    while(list.current() != NULL)
    {
	if(rnd.chance((int) list.current()->size(), (int) size))
	    return list.current();
	else
	    size -= list.current()->size();

	list.next();
    }
    return NULL;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
