// @ myth - server - pack.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef PACK_HEADER
#define PACK_HEADER

// @@ include

#include "server/invent.h"
#include "server/ability.h"

struct BUNCH;

// @@ PACK

class PACK : public INVENTORY
{
    // Keep track of which objects is worn
    //
private:
    ID_LIST worn_list;
    void RemoveObject();
    void AddObject(OBJECT* object);
public:
    bool worn(OBJECT* object);
    void wear(OBJECT* object);
    void TakeOff(OBJECT* object);
    OBJECT* weapon();
    OBJECT* armor();
protected:
    bool MayAdd(OBJECT* object)
    { return !worn(object); }
    //
    // Hold objects to be delete'ed
    //
private:
    OBJECT_LIST junk;
public:
    void destroy(OBJECT* object); // Insert object in junk list
    //
    // Update
    //
private:
    bool TouchPack;
    COUNTER_LIST ability;
    int NumberOfRings;
public:
    int CurrentRing()
    { return NumberOfRings; }
    void AddAbility(COUNTER* counter)
    { ability.insert(counter); }
    bool CheckAbility(ability_type type)
    { return ability.check(type); }
    int GetValue(ability_type type)
    { return ability.value(type); }
    void SendPack(PLAYER* player, force flag = CHECK);
    void update(LEVEL* level, coor pos, MONSTER* monster);
    //
    // Misc
    //
public:
    damage_type absorb(int* power, OBJECT* object,
		       MONSTER* attack, MONSTER* defence);
    void RemoveHalf(SPECIFY* specify, OBJECT_LIST* list, long argument = 0L);
    void DropDead(BUNCH* bunch);
    OBJECT* FindWorst(MONSTER* monster);
    //
    // Administration
    //
private:
    void create();
public:
    void save(BINARY* binary);
    void load(BINARY* binary);
    PACK();
    PACK(PARSE* parse, coor pos);
    ~PACK();
};

// @@ end

#endif PACK_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
