// @ myth - server - scroll.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef SCROLL_HEADER
#define SCROLL_HEADER

// @@ include

#include "server/thing.h"

// @@ SCROLL_BLANK_PAPER

class SCROLL_BLANK_PAPER : public SCROLL
{
    // Identification
    //
protected:
    int BaseValue()
    { return 10; }
public:
    object_type type()
    { return SCROLL_OF_BLANK_PAPER; }
    const char* MagicName()
    { return "blank paper"; }
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new SCROLL_BLANK_PAPER(this, arg); }
    SCROLL_BLANK_PAPER(BINARY* binary) : (binary) { }
    SCROLL_BLANK_PAPER(PARSE* parse) : (parse) { }
    SCROLL_BLANK_PAPER(SCROLL_BLANK_PAPER* thing,
			 long count) : (thing, count) { }
    SCROLL_BLANK_PAPER(long count = 1L) : (count) { }
};

// @@ SCROLL_KNOWLEDGE

class SCROLL_KNOWLEDGE : public SCROLL
{
    // Identification
    //
public:
    object_type type()
    { return SCROLL_OF_KNOWLEDGE; }
    const char* MagicName()
    { return "knowledge"; }
protected:
    int BaseValue()
    { return 25; }
    //
    // Use
    //
private:
    cont CheckIt(MONSTER* monster);
public:
    void effect(LEVEL*, MONSTER* monster);
    QUESTION* DoEffect(LEVEL* level, PLAYER* player, CHANNEL* channel);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new SCROLL_KNOWLEDGE(this, arg); }
    SCROLL_KNOWLEDGE(BINARY* binary) : (binary) { }
    SCROLL_KNOWLEDGE(PARSE* parse) : (parse) { }
    SCROLL_KNOWLEDGE(SCROLL_KNOWLEDGE* thing,
			 long count) : (thing, count) { }
    SCROLL_KNOWLEDGE(long count = 1L) : (count) { }
};

// @@ SCROLL_PRAYERS

class SCROLL_PRAYERS : public SCROLL
{
    // Identification
    //
public:
    object_type type()
    { return SCROLL_OF_PRAYERS; }
    const char* MagicName()
    { return "prayers"; }
protected:
    int BaseValue()
    { return 350; }
    //
    // Use
    //
private:
    cont CheckIt(MONSTER* monster);
    void pray(MONSTER* monster);
public:
    void effect(LEVEL*, MONSTER* monster);
    QUESTION* DoEffect(LEVEL* level, PLAYER* player, CHANNEL* channel);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new SCROLL_PRAYERS(this, arg); }
    SCROLL_PRAYERS(BINARY* binary) : (binary) { }
    SCROLL_PRAYERS(PARSE* parse) : (parse) { }
    SCROLL_PRAYERS(SCROLL_PRAYERS* thing,
			 long count) : (thing, count) { }
    SCROLL_PRAYERS(long count = 1L) : (count) { }
};

// @@ SCROLL_MAGIC

class SCROLL_MAGIC : public SCROLL
{
    // Identification
    //
public:
    object_type type()
    { return SCROLL_OF_MAGIC; }
    const char* MagicName()
    { return "magic"; }
protected:
    int BaseValue()
    { return 300; }
    //
    // Use
    //
private:
    cont CheckIt(MONSTER* monster);
public:
    void effect(LEVEL*, MONSTER* monster);
    QUESTION* DoEffect(LEVEL* level, PLAYER* player, CHANNEL* channel);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new SCROLL_MAGIC(this, arg); }
    SCROLL_MAGIC(BINARY* binary) : (binary) { }
    SCROLL_MAGIC(PARSE* parse) : (parse) { }
    SCROLL_MAGIC(SCROLL_MAGIC* thing,
			 long count) : (thing, count) { }
    SCROLL_MAGIC(long count = 1L) : (count) { }
};

// @@ SCROLL_TELEPORT

class SCROLL_TELEPORT : public SCROLL
{
    // Identification
    //
public:
    object_type type()
    { return SCROLL_OF_TELEPORT; }
    const char* MagicName()
    { return "teleport"; }
protected:
    int BaseValue()
    { return 50; }
    //
    // Use
    //
private:
    cont CheckIt(LEVEL* level, MONSTER* monster);
    void teleport(LEVEL* level, MONSTER* monster);
public:
    void effect(LEVEL* level, MONSTER* monster);
    QUESTION* DoEffect(LEVEL* level, PLAYER* player, CHANNEL* channel);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new SCROLL_TELEPORT(this, arg); }
    SCROLL_TELEPORT(BINARY* binary) : (binary) { }
    SCROLL_TELEPORT(PARSE* parse) : (parse) { }
    SCROLL_TELEPORT(SCROLL_TELEPORT* thing,
			 long count) : (thing, count) { }
    SCROLL_TELEPORT(long count = 1L) : (count) { }
};

// @@ SCROLL_MAPPING

class SCROLL_MAPPING : public SCROLL
{
    // Identification
    //
public:
    object_type type()
    { return SCROLL_OF_MAPPING; }
    const char* MagicName()
    { return "mapping"; }
protected:
    int BaseValue()
    { return 50; }
    //
    // Use
    //
public:
    void effect(LEVEL* level, MONSTER* monster);
    QUESTION* DoEffect(LEVEL* level, PLAYER* player, CHANNEL* channel);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new SCROLL_MAPPING(this, arg); }
    SCROLL_MAPPING(BINARY* binary) : (binary) { }
    SCROLL_MAPPING(PARSE* parse) : (parse) { }
    SCROLL_MAPPING(SCROLL_MAPPING* thing,
			 long count) : (thing, count) { }
    SCROLL_MAPPING(long count = 1L) : (count) { }
};

// @@ end

#endif SCROLL_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
